@@ -244,7 +244,7 @@ class alignas(64) BucketTable {
   // Slightly higher than 1.0, in case hashing to shards isn't
   // perfectly balanced, reserve(size) will still work without
   // rehashing.
-  static constexpr float kDefaultLoadFactor = 1.05;
+  static constexpr float kDefaultLoadFactor = 1.05f;
   typedef std::pair<const KeyType, ValueType> value_type;
 
   using Node =
@@ -266,7 +266,8 @@ class alignas(64) BucketTable {
          (folly::popcount(max_size_ - 1) + ShardBits <= 32)));
     auto buckets = Buckets::create(initial_buckets, cohort);
     buckets_.store(buckets, std::memory_order_release);
-    load_factor_nodes_ = initial_buckets * load_factor_;
+    load_factor_nodes_ =
+        to_integral(static_cast<float>(initial_buckets) * load_factor_);
     bucket_count_.store(initial_buckets, std::memory_order_relaxed);
   }
 
@@ -332,7 +333,8 @@ class alignas(64) BucketTable {
     auto buckets = buckets_.load(std::memory_order_relaxed);
     DCHECK(buckets); // Use-after-destruction by user.
     auto newbuckets = Buckets::create(bucket_count, cohort);
-    load_factor_nodes_ = bucket_count * load_factor_;
+    load_factor_nodes_ =
+        to_integral(static_cast<float>(bucket_count) * load_factor_);
     for (size_t i = 0; i < oldcount; i++) {
       auto bucket = &buckets->buckets_[i]();
       auto node = bucket->load(std::memory_order_relaxed);
@@ -462,11 +464,12 @@ class alignas(64) BucketTable {
   }
 
   void clear(hazptr_obj_cohort<Atom>* cohort) {
-    size_t bcount = bucket_count_.load(std::memory_order_relaxed);
+    size_t bcount;
     Buckets* buckets;
-    auto newbuckets = Buckets::create(bcount, cohort);
     {
       std::lock_guard<Mutex> g(m_);
+      bcount = bucket_count_.load(std::memory_order_relaxed);
+      auto newbuckets = Buckets::create(bcount, cohort);
       buckets = buckets_.load(std::memory_order_relaxed);
       buckets_.store(newbuckets, std::memory_order_release);
       clearSize();
@@ -1405,11 +1408,12 @@ class alignas(64) SIMDTable {
   }
 
   void clear(hazptr_obj_cohort<Atom>* cohort) {
-    size_t ccount = chunk_count_.load(std::memory_order_relaxed);
+    size_t ccount;
     Chunks* chunks;
-    auto newchunks = Chunks::create(ccount, cohort);
     {
       std::lock_guard<Mutex> g(m_);
+      ccount = chunk_count_.load(std::memory_order_relaxed);
+      auto newchunks = Chunks::create(ccount, cohort);
       chunks = chunks_.load(std::memory_order_relaxed);
       chunks_.store(newchunks, std::memory_order_release);
       clearSize();
@@ -1554,7 +1558,8 @@ class alignas(64) SIMDTable {
     }
     auto new_chunks = Chunks::create(new_chunk_count, cohort);
     auto old_chunks = chunks_.load(std::memory_order_relaxed);
-    grow_threshold_ = new_chunk_count * Chunk::kCapacity * load_factor_;
+    grow_threshold_ =
+        to_integral(new_chunk_count * Chunk::kCapacity * load_factor_);
 
     for (size_t i = 0; i < old_chunk_count; i++) {
       DCHECK(old_chunks); // Use-after-destruction by user.
@@ -1792,10 +1797,9 @@ class alignas(64) ConcurrentHashMapSegment {
     }
     return res;
   }
-
-  template <typename Key, typename Value>
-  bool assign_if_equal(
-      Iterator& it, Key&& k, const ValueType& expected, Value&& desired) {
+  template <typename Key, typename Value, typename Predicate>
+  bool assign_if(
+      Iterator& it, Key&& k, Value&& desired, Predicate&& predicate) {
     auto node = (Node*)Allocator().allocate(sizeof(Node));
     new (node)
         Node(cohort_, std::forward<Key>(k), std::forward<Value>(desired));
@@ -1803,7 +1807,7 @@ class alignas(64) ConcurrentHashMapSegment {
         it,
         node->getItem().first,
         InsertType::MATCH,
-        [&expected](const ValueType& v) { return v == expected; },
+        std::forward<Predicate>(predicate),
         node);
     if (!res) {
       node->~Node();
@@ -1812,6 +1816,16 @@ class alignas(64) ConcurrentHashMapSegment {
     return res;
   }
 
+  template <typename Key, typename Value>
+  bool assign_if_equal(
+      Iterator& it, Key&& k, const ValueType& expected, Value&& desired) {
+    return assign_if(
+        it,
+        std::forward<Key>(k),
+        std::forward<Value>(desired),
+        [&expected](const ValueType& v) { return v == expected; });
+  }
+
   template <typename MatchFunc, typename K, typename... Args>
   bool insert_internal(
       Iterator& it,
