@@ -1249,10 +1249,13 @@ REGISTER_TYPED_TEST_SUITE_P(
     RunInEventBaseThreadAndWait,
     RunImmediatelyOrRunInEventBaseThreadAndWaitCross,
     RunImmediatelyOrRunInEventBaseThreadAndWaitWithin,
+    RunImmediatelyOrRunInEventBaseThreadAndWaitNotLooping,
+    RunImmediatelyOrRunInEventBaseThreadCross,
     RunImmediatelyOrRunInEventBaseThreadNotLooping,
     RepeatedRunInLoop,
     RunInLoopNoTimeMeasurement,
     RunInLoopStopLoop,
+    RunPoolLoop,
     messageAvailableException,
     TryRunningAfterTerminate,
     CancelRunInLoop,
