@@ -14,6 +14,7 @@
  * limitations under the License.
  */
 
+#include <folly/io/SocketOptionMap.h>
 #include <folly/io/async/AsyncUDPSocket.h>
 
 #include <cerrno>
@@ -160,6 +161,26 @@ void AsyncUDPSocket::init(sa_family_t family, BindOptions bindOptions) {
     }
   }
 
+  if (transparent_) {
+    int optname = 0;
+#if defined(IP_TRANSPARENT)
+    optname = IP_TRANSPARENT;
+#endif
+    if (!optname) {
+      throw AsyncSocketException(
+          AsyncSocketException::NOT_OPEN, "IP_TRANSPARENT is not supported");
+    }
+    // set the socket IP transparent mode
+    int value = 1;
+    if (netops::setsockopt(
+            socket, IPPROTO_IP, optname, &value, sizeof(value)) != 0) {
+      throw AsyncSocketException(
+          AsyncSocketException::NOT_OPEN,
+          "failed to set socket IP transparent mode",
+          errno);
+    }
+  }
+
   if (busyPollUs_ > 0) {
     int optname = 0;
 #if defined(SO_BUSY_POLL)
@@ -480,6 +501,7 @@ ssize_t AsyncUDPSocket::writeChain(
     const folly::SocketAddress& address,
     std::unique_ptr<folly::IOBuf>&& buf,
     WriteOptions options) {
+  CHECK(nontrivialCmsgs_.empty()) << "Nontrivial options are not supported";
   int msg_flags = options.zerocopy ? getZeroCopyFlags() : 0;
   iovec vec[16];
   size_t iovec_len = buf->fillIov(vec, sizeof(vec) / sizeof(vec[0])).numIovecs;
@@ -616,7 +638,7 @@ ssize_t AsyncUDPSocket::writev(
   controlBufSize +=
       cmsgs_.size() * (CMSG_SPACE(sizeof(int)) / CMSG_SPACE(sizeof(uint16_t)));
 
-  if (controlBufSize <= kSmallSizeMax) {
+  if (nontrivialCmsgs_.empty() && controlBufSize <= kSmallSizeMax) {
     // suppress "warning: variable length array 'control' is used [-Wvla]"
     FOLLY_PUSH_WARNING
     FOLLY_GNU_DISABLE_WARNING("-Wvla")
@@ -629,9 +651,12 @@ ssize_t AsyncUDPSocket::writev(
     FOLLY_POP_WARNING
     return writevImpl(&msg, gso);
   } else {
-    std::unique_ptr<char[]> control(
-        new char[controlBufSize * (CMSG_SPACE(sizeof(uint16_t)))]);
-    memset(control.get(), 0, controlBufSize * (CMSG_SPACE(sizeof(uint16_t))));
+    controlBufSize *= CMSG_SPACE(sizeof(uint16_t));
+    for (const auto& itr : nontrivialCmsgs_) {
+      controlBufSize += CMSG_SPACE(itr.second.size());
+    }
+    std::unique_ptr<char[]> control(new char[controlBufSize]);
+    memset(control.get(), 0, controlBufSize);
     msg.msg_control = control.get();
     return writevImpl(&msg, gso);
   }
@@ -662,6 +687,22 @@ ssize_t AsyncUDPSocket::writevImpl(
       memcpy(CMSG_DATA(cm), &val, sizeof(val));
     }
   }
+  for (const auto& itr : nontrivialCmsgs_) {
+    const auto& key = itr.first;
+    const auto& val = itr.second;
+    msg->msg_controllen += CMSG_SPACE(val.size());
+    if (cm) {
+      cm = CMSG_NXTHDR(msg, cm);
+    } else {
+      cm = CMSG_FIRSTHDR(msg);
+    }
+    if (cm) {
+      cm->cmsg_level = key.level;
+      cm->cmsg_type = key.optname;
+      cm->cmsg_len = CMSG_LEN(val.size());
+      memcpy(CMSG_DATA(cm), val.data(), val.size());
+    }
+  }
 
   if (gso > 0) {
     msg->msg_controllen += CMSG_SPACE(sizeof(uint16_t));
@@ -715,7 +756,7 @@ int AsyncUDPSocket::writemGSO(
   singleControlBufSize +=
       cmsgs_.size() * (CMSG_SPACE(sizeof(int)) / CMSG_SPACE(sizeof(uint16_t)));
   size_t controlBufSize = count * singleControlBufSize;
-  if (controlBufSize <= kSmallSizeMax) {
+  if (nontrivialCmsgs_.empty() && controlBufSize <= kSmallSizeMax) {
     // suppress "warning: variable length array 'vec' is used [-Wvla]"
     FOLLY_PUSH_WARNING
     FOLLY_GNU_DISABLE_WARNING("-Wvla")
@@ -733,9 +774,12 @@ int AsyncUDPSocket::writemGSO(
   } else {
     std::unique_ptr<mmsghdr[]> vec(new mmsghdr[count]);
 #ifdef FOLLY_HAVE_MSG_ERRQUEUE
-    std::unique_ptr<char[]> control(
-        new char[controlBufSize * (CMSG_SPACE(sizeof(uint16_t)))]);
-    memset(control.get(), 0, controlBufSize * (CMSG_SPACE(sizeof(uint16_t))));
+    controlBufSize *= (CMSG_SPACE(sizeof(uint16_t)));
+    for (const auto& itr : nontrivialCmsgs_) {
+      controlBufSize += CMSG_SPACE(itr.second.size());
+    }
+    std::unique_ptr<char[]> control(new char[controlBufSize]);
+    memset(control.get(), 0, controlBufSize);
     controlPtr = control.get();
 #endif
     ret = writeImpl(addrs, bufs, count, vec.get(), gso, controlPtr);
@@ -801,6 +845,23 @@ void AsyncUDPSocket::fillMsgVec(
         memcpy(CMSG_DATA(cm), &val, sizeof(val));
       }
     }
+    for (const auto& itr : nontrivialCmsgs_) {
+      const auto& key = itr.first;
+      const auto& val = itr.second;
+      msg.msg_controllen += CMSG_SPACE(val.size());
+      if (cm) {
+        cm = CMSG_NXTHDR(&msg, cm);
+      } else {
+        cm = CMSG_FIRSTHDR(&msg);
+      }
+      if (cm) {
+        cm->cmsg_level = key.level;
+        cm->cmsg_type = key.optname;
+        cm->cmsg_len = CMSG_LEN(val.size());
+        memcpy(CMSG_DATA(cm), val.data(), val.size());
+      }
+    }
+
     // handle GSO
     if (gso && gso[i] > 0) {
       msg.msg_controllen += CMSG_SPACE(sizeof(uint16_t));
@@ -1371,6 +1432,17 @@ void AsyncUDPSocket::applyOptions(
   }
 }
 
+void AsyncUDPSocket::applyNontrivialOptions(
+    const SocketNontrivialOptionMap& options, SocketOptionKey::ApplyPos pos) {
+  auto result = applySocketOptions(fd_, options, pos);
+  if (result) {
+    throw AsyncSocketException(
+        AsyncSocketException::INTERNAL_ERROR,
+        "failed to set nontrivial socket option",
+        result);
+  }
+}
+
 void AsyncUDPSocket::detachEventBase() {
   DCHECK(eventBase_ && eventBase_->isInEventBaseThread());
   registerHandler(uint16_t(NONE));
@@ -1390,10 +1462,23 @@ void AsyncUDPSocket::setCmsgs(const SocketOptionMap& cmsgs) {
   cmsgs_ = cmsgs;
 }
 
+void AsyncUDPSocket::setNontrivialCmsgs(
+    const SocketNontrivialOptionMap& nontrivialCmsgs) {
+  nontrivialCmsgs_ = nontrivialCmsgs;
+}
+
 void AsyncUDPSocket::appendCmsgs(const SocketOptionMap& cmsgs) {
   for (auto itr = cmsgs.begin(); itr != cmsgs.end(); ++itr) {
     cmsgs_[itr->first] = itr->second;
   }
 }
 
+void AsyncUDPSocket::appendNontrivialCmsgs(
+    const SocketNontrivialOptionMap& nontrivialCmsgs) {
+  for (auto itr = nontrivialCmsgs.begin(); itr != nontrivialCmsgs.end();
+       ++itr) {
+    nontrivialCmsgs_[itr->first] = itr->second;
+  }
+}
+
 } // namespace folly
