@@ -1487,7 +1487,8 @@ TYPED_TEST_P(EventBaseTest, RunImmediatelyOrRunInEventBaseThreadAndWaitWithin) {
   });
 }
 
-TYPED_TEST_P(EventBaseTest, RunImmediatelyOrRunInEventBaseThreadNotLooping) {
+TYPED_TEST_P(
+    EventBaseTest, RunImmediatelyOrRunInEventBaseThreadAndWaitNotLooping) {
   auto evbPtr = getEventBase<TypeParam>();
   SKIP_IF(!evbPtr) << "Backend not available";
   folly::EventBase& eb = *evbPtr;
@@ -1496,6 +1497,38 @@ TYPED_TEST_P(EventBaseTest, RunImmediatelyOrRunInEventBaseThreadNotLooping) {
   EXPECT_TRUE(mutated);
 }
 
+TYPED_TEST_P(EventBaseTest, RunImmediatelyOrRunInEventBaseThreadCross) {
+  auto evbPtr = getEventBase<TypeParam>();
+  SKIP_IF(!evbPtr) << "Backend not available";
+  folly::EventBase& eb = *evbPtr;
+  std::thread th(&EventBase::loopForever, &eb);
+  SCOPE_EXIT {
+    eb.terminateLoopSoon();
+    th.join();
+  };
+  // wait for loop to start
+  eb.runInEventBaseThreadAndWait([] {});
+  Baton<> baton1, baton2;
+  EXPECT_FALSE(eb.isInEventBaseThread());
+
+  eb.runImmediatelyOrRunInEventBaseThread([&] {
+    baton1.wait();
+    baton2.post();
+  });
+  EXPECT_FALSE(baton2.ready());
+  baton1.post();
+  EXPECT_TRUE(baton2.try_wait_for(std::chrono::milliseconds(100)));
+}
+
+TYPED_TEST_P(EventBaseTest, RunImmediatelyOrRunInEventBaseThreadNotLooping) {
+  auto evbPtr = getEventBase<TypeParam>();
+  SKIP_IF(!evbPtr) << "Backend not available";
+  folly::EventBase& eb = *evbPtr;
+  Baton<> baton;
+  eb.runImmediatelyOrRunInEventBaseThread([&] { baton.post(); });
+  EXPECT_TRUE(baton.ready());
+}
+
 ///////////////////////////////////////////////////////////////////////////
 // Tests for runInLoop()
 ///////////////////////////////////////////////////////////////////////////
@@ -1594,6 +1627,38 @@ TYPED_TEST_P(EventBaseTest, RunInLoopStopLoop) {
   ASSERT_LE(c1.getCount(), 11);
 }
 
+// Test loopPool() call sequence
+TYPED_TEST_P(EventBaseTest, RunPoolLoop) {
+  auto evbPtr = getEventBase<TypeParam>();
+  SKIP_IF(!evbPtr) << "Backend not available";
+  folly::EventBase& eventBase = *evbPtr;
+  std::atomic<bool> running = true;
+  int calls = 0;
+
+  CountedLoopCallback c1(&eventBase, 20);
+  CountedLoopCallback c2(
+      &eventBase, 10, [eb = &eventBase, running = &running]() {
+        eb->terminateLoopSoon();
+        running->store(false);
+      });
+
+  eventBase.runInLoop(&c1);
+  eventBase.runInLoop(&c2);
+  ASSERT_EQ(c1.getCount(), 20);
+  ASSERT_EQ(c2.getCount(), 10);
+
+  eventBase.loopPollSetup();
+  while (running.load()) {
+    calls++;
+    eventBase.loopPoll();
+  }
+  eventBase.loopPollCleanup();
+
+  // We expect multiple iterations of the loop to happen, since loopPool has non
+  // blocking semantics, we should call loopPool multiple times
+  ASSERT_GT(calls, 1);
+}
+
 TYPED_TEST_P(EventBaseTest1, pidCheck) {
   auto evbPtr = getEventBase<TypeParam>();
   SKIP_IF(!evbPtr) << "Backend not available";
