@@ -389,16 +389,9 @@ TEST(SimpleSubprocessTest, FromExistingProcess) {
 }
 
 TEST(ParentDeathSubprocessTest, ParentDeathSignal) {
-  // Find out where we are.
-  const auto basename = "subprocess_test_parent_death_helper";
-  auto helper = fs::executable_path();
-  helper.remove_filename() /= basename;
-  if (!fs::exists(helper)) {
-    helper = helper.parent_path().parent_path() / basename / basename;
-  }
-
+  auto helper = folly::test::find_resource(
+      "folly/test/subprocess_test_parent_death_helper");
   fs::path tempFile(fs::temp_directory_path() / fs::unique_path());
-
   std::vector<std::string> args{helper.string(), tempFile.string()};
   Subprocess proc(args);
   // The helper gets killed by its child, see details in
