@@ -40,14 +40,23 @@
 
 #include <folly/Memory.h>
 #include <folly/experimental/io/FsUtil.h>
+#include <folly/lang/Keep.h>
 #include <folly/portability/GTest.h>
 #include <folly/portability/Unistd.h>
 #include <folly/synchronization/Baton.h>
-#include <folly/synchronization/detail/ThreadCachedInts.h>
 #include <folly/system/ThreadId.h>
 
 using namespace folly;
 
+extern "C" FOLLY_KEEP int* check_thread_local_get(ThreadLocal<int>& o) {
+  return o.get();
+}
+
+extern "C" FOLLY_KEEP int* check_thread_local_get_existing(
+    ThreadLocal<int>& o) {
+  return o.get_existing();
+}
+
 struct Widget {
   static int totalVal_;
   static int totalMade_;
@@ -242,7 +251,7 @@ TEST(ThreadLocal, GetWithoutCreateUncreated) {
   Widget::totalMade_ = 0;
   ThreadLocal<Widget> w;
   std::thread([&w]() {
-    auto ptr = w.getIfExist();
+    auto ptr = w.get_existing();
     if (ptr) {
       ptr->val_++;
     }
@@ -256,7 +265,7 @@ TEST(ThreadLocal, GetWithoutCreateGets) {
   ThreadLocal<Widget> w;
   std::thread([&w]() {
     w->val_++;
-    auto ptr = w.getIfExist();
+    auto ptr = w.get_existing();
     if (ptr) {
       ptr->val_++;
     }
@@ -437,36 +446,6 @@ TEST(ThreadLocal, Movable2) {
   EXPECT_EQ(4, tls.size());
 }
 
-namespace {
-class ThreadCachedIntWidget {
- public:
-  ThreadCachedIntWidget() {}
-
-  ~ThreadCachedIntWidget() {
-    if (ints_) {
-      ints_->increment(0);
-    }
-  }
-
-  void set(detail::ThreadCachedInts<void>* ints) { ints_ = ints; }
-
- private:
-  detail::ThreadCachedInts<void>* ints_{nullptr};
-};
-} // namespace
-
-TEST(ThreadLocal, TCICreateOnThreadExit) {
-  detail::ThreadCachedInts<void> ints;
-  ThreadLocal<ThreadCachedIntWidget> w;
-
-  std::thread([&] {
-    // make sure the ints object is created
-    ints.increment(1);
-    // now the widget
-    w->set(&ints);
-  }).join();
-}
-
 namespace {
 
 constexpr size_t kFillObjectSize = 300;
