@@ -18,8 +18,33 @@
 
 #include <type_traits>
 
+#include <folly/lang/Keep.h>
 #include <folly/portability/GTest.h>
 
+namespace folly {
+
+extern "C" FOLLY_KEEP int check_unsafe_default_initialized_int_ret() {
+  int a = folly::unsafe_default_initialized;
+  return a;
+}
+
+extern "C" FOLLY_KEEP void check_unsafe_default_initialized_int_set(int* p) {
+  int a = folly::unsafe_default_initialized;
+  *p = a;
+}
+
+extern "C" FOLLY_KEEP void check_unsafe_default_initialized_int_pass() {
+  int a = folly::unsafe_default_initialized;
+  folly::detail::keep_sink_nx(a);
+}
+
+extern "C" FOLLY_KEEP int check_unsafe_default_initialized_int_cexpr() {
+  constexpr int a = folly::unsafe_default_initialized;
+  return a;
+}
+
+} // namespace folly
+
 namespace {
 
 class UtilityTest : public testing::Test {};
@@ -120,6 +145,20 @@ TEST_F(UtilityTest, forward_like) {
   // folly::forward_like<const int&>(1);
 }
 
+TEST_F(UtilityTest, inheritable) {
+  struct foo : folly::index_constant<47> {};
+  struct bar final : folly::index_constant<89> {};
+  using ifoo = folly::detail::inheritable<foo>;
+  using ibar = folly::detail::inheritable<bar>;
+  EXPECT_TRUE(std::is_empty_v<ifoo>);
+  EXPECT_FALSE(std::is_empty_v<ibar>);
+  struct tester : ifoo, ibar {};
+  EXPECT_EQ(47, static_cast<foo const&>(tester{}));
+  EXPECT_EQ(89, static_cast<bar const&>(tester{}));
+  EXPECT_EQ(47, static_cast<foo const&>(folly::copy(tester{})));
+  EXPECT_EQ(89, static_cast<bar const&>(folly::copy(tester{})));
+}
+
 TEST_F(UtilityTest, MoveOnly) {
   class FooBar : folly::MoveOnly {
     int a;
@@ -173,3 +212,17 @@ TEST_F(UtilityTest, to_narrow) {
     EXPECT_EQ(100, actual);
   }
 }
+
+TEST_F(UtilityTest, to_integral) {
+  {
+    constexpr uint32_t actual = folly::to_integral(100.0f);
+    EXPECT_EQ(100, actual);
+  }
+}
+
+TEST_F(UtilityTest, to_floating_point) {
+  {
+    constexpr float actual = folly::to_floating_point(100);
+    EXPECT_EQ(100.f, actual);
+  }
+}
